from datetime import datetime

import json
import pytz
from dateutil.parser import parse
from django.http import JsonResponse
from rest_framework import generics
from rest_framework.decorators import api_view, permission_classes
from rest_framework.permissions import IsAuthenticated

from clusterCore.api.serializer import get_create_node_response
from clusterCore.enums import DBType, Inventory, ClusterRoles, Status, Schedule, TaskState
from clusterCore.exceptions import *
from c123_kubernetes.tasks import drop_mongo_k8s_pod, export_mongo_k8s_node
from clusterCore.models import c123_node, c123_inventory, Task
from clusterCore.scheduler import MISFIRE_GRACE_TIME_BACKUP
from clusterCore.tasks import (append_status_cmd, get_cron_schedule, get_backup_description, c123_HostSerializer, object_to_json)
from .serializer import c123_CreateMongoPodSerializer


class c123_k8s_CreateMongoPodView(generics.CreateAPIView):
    permission_classes = [IsAuthenticated]
    queryset = c123_node.objects.all()
    serializer_class = c123_CreateMongoPodSerializer
    def create(self, request, *args, **kwargs):
        return get_create_node_response(self, request, *args, **kwargs)

@api_view(['POST'])
@permission_classes((IsAuthenticated,))
def c123_k8s_DropMongoPodView(request):
    node_id = request.data.get("node_id")
    api_call = request.data.get("rest_api")
    final_backup = request.data.get('final_backup')
    if api_call:
        node = c123_node.objects.filter(type=DBType.MONGO_K8S.value, name=node_id, ind_deleted=False).first()
    else:
        node = c123_node.objects.filter(type=DBType.MONGO_K8S.value, pk=node_id, ind_deleted=False).first()
    if not node:
        raise APIException410(f"node id {node_id} not available")

    task = drop_mongo_k8s_pod(node, final_backup)
    return append_status_cmd(request, task)

@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def DumpMongoNode_k8s(request):
    """dump for mongo node"""
    data = request.data
    data['created_by'] = request.user.id
    schedule = data.get('schedule')
    api_call = data.get('api_call')
    node_id = data.get('node_id')

    if not schedule:
        task = export_mongo_k8s_node(data)
        return append_status_cmd(request, task)
    else:
        from django.conf import settings
        client_timezone = pytz.timezone(data.get("client_timezone"))
        schedule_run_time_str = data.get('schedule_run_time')
        schedule_run_time = parse(schedule_run_time_str).astimezone(client_timezone)
        client_hours = schedule_run_time.hour
        client_minutes = schedule_run_time.minute
        client_seconds = schedule_run_time.second
        days_of_week = data.get("days_of_week")
        days_of_month = data.get("days_of_month")
        if days_of_month:
            int_days_of_month = [parse(d).astimezone(client_timezone).day for d in days_of_month]
        else:
            int_days_of_month = None
        description = get_backup_description(client_hours, client_minutes, client_seconds, days_of_week, int_days_of_month)
        localized_datetime = datetime.now(client_timezone).replace(hour=client_hours, minute=client_minutes, second=client_seconds).astimezone(pytz.timezone(settings.TIME_ZONE))
        hours = localized_datetime.hour
        minutes = localized_datetime.minute
        seconds = localized_datetime.second
        trigger = get_cron_schedule(hours, minutes, seconds, days_of_week, int_days_of_month)
        if api_call:
            node = c123_node.objects.filter(type = DBType.MONGO.value, name=node_id, ind_deleted = False).first()
        else:
            node = c123_node.objects.filter(type = DBType.MONGO.value, pk=node_id, ind_deleted = False).first()
        if not node:
            raise APIException410("node id {} not available".format(node_id))
        job_id =  f"export_{node.name}"
        scheduler = settings.scheduler
        export_job = next((i for i in scheduler.get_jobs() if job_id in i.id), None)
        if export_job:
            export_job.reschedule(trigger=trigger)
        else:
             settings.scheduler.add_job(export_mongo_k8s_node, replace_existing=True, trigger=trigger,  kwargs={"data": data}, id=job_id, misfire_grace_time=MISFIRE_GRACE_TIME_BACKUP)
        return JsonResponse({"notification": f"export {node.name} {description} started"}, status = 200)


@api_view(['POST'])
@permission_classes((IsAuthenticated,))
def RestoreDumpMongoNode_k8s(request):
    """restore mongo node from dump backup
    the call is made from client AJAX POST that contains the dump id"""
    dump_id = request.data.get('dump_id')
    api_call = request.data.get('rest_api')
    if api_call:
        dump = c123_inventory.objects.filter(name=dump_id, type=Inventory.EXPORT.value).first()
    else:
        dump = c123_inventory.objects.filter(pk=dump_id, type=Inventory.EXPORT.value).first()
    if not dump:
        raise APIException404(f"dump id {dump_id} not found")
    # node = dump.node
    # if node.cluster_role == ClusterRoles.ARBITER.value:
    #     raise APIException403("dump is not allowed for arbiter")
    # path = dump.ex_params.get('backup_path')
    # app_host = c123_host.objects.filter(ex_params__myself=True).first()
    # if not app_host:
    #     raise APIException404("could not determine the app host")
    # path_exists = checkPathExists(app_host, path)
    # if not path_exists:
    #     raise APIException404(f"{path}: no such file or directory")
    node = dump.node
    primary = c123_node.objects.filter(cluster=node.cluster, type=DBType.MONGO_K8S.value,
                                       cluster_role=ClusterRoles.PRIMARY.value, ind_deleted=False).first()
    if not primary:
        raise APIException404("primary not found")

    # is_master = find_replicaset_primary(primary) == primary.name
    # if not is_master:
    #     nodes = c123_node.objects.filter(type=DBType.MONGO.value, ind_deleted=False, cluster=primary.cluster).exclude(
    #         pk=primary.pk)
    #
    #     for cluster_node in nodes:
    #         master = find_replicaset_primary(cluster_node)
    #
    #         if master != 'none':
    #             primary = c123_node.objects.filter(type=DBType.MONGO.value, cluster=primary.cluster, name=master,
    #                                                ind_deleted=False).first()
    #             if not primary:
    #                 raise APIException410("primary not in app db")
    #             primary.cluster_role = ClusterRoles.PRIMARY.value
    #             primary.ex_params["mongo_master_id"] = primary.pk
    #             primary.save()
    #             others = c123_node.objects.filter(type=DBType.MONGO.value, ind_deleted=False, cluster=node.cluster,
    #                                               ex_params__replicaset_name=primary.ex_params.get('replicaset_name'))
    #
    #             for other in others:
    #                 if other.cluster_role == ClusterRoles.PRIMARY.value:
    #                     other.cluster_role = ClusterRoles.STANDBY.value
    #                 other.ex_params["mongo_master_id"] = primary.pk
    #                 other.save()
    #             break

    primary.status = Status.RESTORING.value
    primary.save()
    # serialize objects for the tasks
    # dump_host = c123_HostSerializer(app_host).data
    dump = object_to_json(dump)
    node_data = object_to_json(primary)
    host_data = c123_HostSerializer(primary.host).data
    created_by = request.user
    tmp_data = json.dumps(
        {'db_name': None, 'node_data': node_data, 'host_data': host_data, 'dump': dump,
         'created_by': created_by.id})
    task = Task.objects.create(task='mongorestore_task',
                               displayment=f'restore mongo cluster {primary.name}',
                               target_type=DBType.MONGO_K8S.value,
                               target_id=primary.id,
                               org_id=primary.org_id,
                               schedule=Schedule.NOW.value,
                               status=TaskState.PENDING.value,
                               parameters=tmp_data,
                               created_by=created_by)
    return append_status_cmd(request, task)
